@@ -37,6 +37,7 @@ from cycler import cycler
 import psutil
 import numpy as np
 import scipy.stats
+import asteval
 
 import rich
 from rich import print
@@ -126,7 +127,7 @@ def regex(s):
         pattern = re.compile(s)
     except Exception as e:
         raise argparse.ArgumentTypeError(
-            f"{pattern!r} is not a valid regular expression: {e}"
+            f"{s!r} is not a valid regular expression: {e}"
         )
     return pattern
 
@@ -175,6 +176,44 @@ def str_with_index_list(s):
     ]
 
 
+groupby_aliases = dict(
+    week='"%Y W%W"',
+    usweek='"%Y W%U"',
+    fortnite='str(%Y) + " fortnite " + "{:02d}".format(int("%W".lstrip("0")) // 2)',
+    usfortnite='str(%Y) + " fortnite " + "{:02d}".format(int("%U".lstrip("0")) // 2)',
+    month='"%Y %b"',
+    monthinyear='"%b"',
+    weekinyear='"%W"',
+    year='"%Y"',
+    quarter='"%YQ" + str((int("%m".lstrip("0")) - 1) // 3 + 1)',
+)
+
+
+def astevaled_strftime(s):
+    s = groupby_aliases.get(s, s)
+    fmt_example = (
+        "Format example: --groupby 'month' "
+        f"or --groupby 'week' "
+        f"or --groupby '%-W // 2' "
+        f"(floor division of week in year), not {s!r}"
+    )
+    aeval = asteval.Interpreter()
+    try:
+        now = pd.to_datetime(
+            datetime.datetime.now()
+        )  # we use pandas dates later, so check here with pandas dates
+        formatted = now.strftime(s)
+        evaled = aeval(formatted)
+        for error in aeval.error:
+            raise error.exc(error.msg)
+    except Exception as e:
+        raise argparse.ArgumentTypeError(
+            f"{s!r} is not an asteval'able strftime format ({type(e).__name__}: {e})\n"
+            f"{fmt_example}"
+        )
+    return s
+
+
 parser = argparse.ArgumentParser(
     description=f"""
 
@@ -212,6 +251,9 @@ version {__version__}, written by Yann Büchau, source code is at https://gitlab
 # „How much did and will I pay for that one house?” (if you tagged house transactions with '; house: MyHouse')
 > hledger-plot balance not:acct:^Assets --historical --daily tag:house=MyHouse --pivot=house --forecast --end 2030
 
+# How was the quarter performance in 2022?
+> hledger-plot balance ^expenses ^revenues --daily --depth 1 date:2022 --groupby quarter
+
 version {__version__}, written by Yann Büchau, source code is at https://gitlab.com/nobodyinperson/hledger-utils
 """.strip(),
 )
@@ -411,11 +453,28 @@ parser_modify_group.add_argument(
     type=regex_with_date_range_and_reference_date,
     default=[],
 )
+parser_modify_group.add_argument(
+    "--groupby",
+    metavar="[INT] PERIOD",
+    help=f"""
+group the transactions into time bins and break those out as separate lines.
+For example --groupby 'quarter' lets you compare quarters or --groupby 'week'.
+The period can also be a simple Python expression where strftime format codes are replaced, e.g. 'str(%Y) + " fortnite " + str(%-W // 2)' for fortnitely grouping or '"%b"' (note the extra quoting to make it a Python string!) to group by month ('"%Y %b"' might make more sense for more years).
+This expression is evaluated safe-ish-ly with asteval.
+Aliases: {str(groupby_aliases)}.
+Can be specified multiple times.
+""".replace(
+        "%", "%%"
+    ).strip(),
+    type=astevaled_strftime,
+    action="append",
+    default=[],
+)
 parser_modify_group.add_argument(
     "--mod-order",
     metavar="LIST",
     default=str_with_index_list(
-        default := "rename invert sum multiply resample mean trend drop only"
+        default := "rename invert sum multiply resample groupby mean trend drop only"
     ),
     help="List of modification steps like 'sum(1),rename(2),resample(1)' "
     "specifying the application order of modifiers. "
@@ -530,6 +589,9 @@ def cli(cli_args=sys.argv[1:]):
             f"Assuming you want to add a --trend for all columns, "
             "setting selector to {args.trend}"
         )
+    if args.groupby:
+        logger.info(f"Hiding 'today' line as it doesn't work with --groupby")
+        args.no_today = True
     logger.debug(f"{hledger_args = }")
 
     try:
@@ -730,6 +792,12 @@ def cli(cli_args=sys.argv[1:]):
             )
         )
         account_commodities[newname] = m.get("commodity", "")
+    if not data_to_concat:
+        logger.error(
+            f"There is no data to plot. "
+            "Check your hledger source files and if your query actually returns something."
+        )
+        sys.exit(1)
     data = pd.concat(
         data_to_concat,
         axis="columns",
@@ -861,6 +929,30 @@ pip install windows-curses
             data = data.resample(interval).sum()
         return data
 
+    def groupby_handler(groupbys, data=data):
+        for fmt in groupbys:
+            logger.debug(f"--groupby: {fmt = }")
+            formatted = data.index.strftime(fmt)
+            logger.debug(
+                f"--groupby: formatted with {fmt!r}:\n{formatted.values}"
+            )
+            groupby = formatted.to_series().apply(asteval.Interpreter()).values
+            logger.debug(f"--groupby: evaluated:\n{groupby}")
+
+            parts = []
+            for n, g in data.groupby(groupby):
+                g = g.rename(
+                    columns={col: f"{col} ({n})" for col in g},
+                )
+                g.index = (
+                    (g.index - g.index.min()) / np.timedelta64("1", "D")
+                ).astype(int)
+                parts.append(g)
+
+            data = pd.concat(parts, axis="columns")
+
+        return data
+
     def trend_handler(trenddefs, data=data):
         for trenddef in trenddefs:
             logger.debug(f"{trenddef = }")
@@ -882,12 +974,15 @@ pip install windows-curses
                 )
             if cols_to_fit := list(filter(pattern.search, data)):
                 for col in cols_to_fit:
-                    series = data.loc[timerange, col]
+                    series = data.loc[timerange, col].dropna()
                     logger.debug(f"series = \n{series}")
-                    days_since_reference = (
-                        series.index.map(pd.Timestamp.timestamp)
-                        - (reference.timestamp() if reference else 0)
-                    ) / (24 * 60 * 60)
+                    if hasattr(series.index, "strftime"):
+                        days_since_reference = (
+                            series.index.map(pd.Timestamp.timestamp)
+                            - (reference.timestamp() if reference else 0)
+                        ) / (24 * 60 * 60)
+                    else:
+                        days_since_reference = series.index.values
                     fit = scipy.stats.linregress(days_since_reference, series)
 
                     def oma(x):
@@ -979,6 +1074,7 @@ pip install windows-curses
         drop=drop_handler,
         only=only_handler,
         resample=resample_handler,
+        groupby=groupby_handler,
         trend=trend_handler,
     )
 
@@ -1011,6 +1107,9 @@ pip install windows-curses
                 f"No handler for --mod-order step {modname = !r} {index = } ({allmodsteps = })"
             )
 
+    logger.debug(f"Sorting data")
+    data = data.sort_index()
+
     logger.info("📈  Plotting...")
     with plt.xkcd() if args.xkcd else nothing():
         fig, ax = plt.subplots(num=args.title)
@@ -1066,7 +1165,8 @@ pip install windows-curses
                         )
                 ax.plot(series.index, series, **plot_kwargs)
                 used_styles.append(only_prop_cycle(plot_kwargs))
-                logger.debug(f"{used_styles = }")
+                if logger.getEffectiveLevel() < logging.DEBUG:
+                    logger.debug(f"{used_styles = }")
 
             if not args.no_today:
                 ax.axvline(
@@ -1082,6 +1182,8 @@ pip install windows-curses
         if args.title:
             fig.suptitle(args.title)
         ax.set_title(args.axtitle or " ".join(hledger_args))
+        if args.groupby:
+            ax.set_xlabel(f"days within period")
         ax.set_ylabel(" ".join(f"[{c}]" for c in commodities))
         ax.legend(ncols=math.ceil(len(ax.get_lines()) / 40))
 
