@@ -14,7 +14,7 @@ from contextlib import contextmanager
 import hledger_utils
 
 # external modules
-
+import rich
 
 logger = logging.getLogger(__name__)
 
@@ -31,51 +31,75 @@ def environment(**env):
             os.environ[k] = v
 
 
-class HledgerEditTest(unittest.TestCase):
-    def assert_hledger_edit(self, content, editor, expected_result, query=""):
-        try:
-            files = {
-                tempfile.mkstemp(
-                    prefix="hledger-edit-test-",
-                    suffix=".hledger",
-                )[1]: c
-                for c in zip(
-                    ([content] if isinstance(content, str) else content),
-                    (
-                        [expected_result]
-                        if isinstance(expected_result, str)
-                        else expected_result
-                    ),
-                )
-            }
+@contextmanager
+def nothing():
+    yield
 
-            for tmpfilePath, (content, expected) in files.items():
-                with open(tmpfilePath, "w") as fh:
-                    fh.write(content)
-                    fh.flush()
-            with environment(
-                EDITOR=editor,
-                VISUAL=editor,
-                HLEDGER_EDIT_LOGLEVEL="CRITICAL",
-                HLEDGER_EDIT_NOTTY="yes",
-            ):
-                hledger_utils.commands.edit.cli(
-                    list(
-                        itertools.chain.from_iterable(("-f", f) for f in files)
+
+class HledgerEditTest(unittest.TestCase):
+    def assert_hledger_edit(
+        self,
+        content,
+        editor,
+        expected_result,
+        query="",
+        allowed_exitcodes=None,
+    ):
+        try:
+            with (
+                nothing()
+                if allowed_exitcodes is None
+                else self.assertRaises(SystemExit)
+            ) as cm:
+                files = {
+                    tempfile.mkstemp(
+                        prefix="hledger-edit-test-",
+                        suffix=".hledger",
+                    )[1]: c
+                    for c in zip(
+                        ([content] if isinstance(content, str) else content),
+                        (
+                            [expected_result]
+                            if isinstance(expected_result, str)
+                            else expected_result
+                        ),
                     )
-                    + shlex.split(query)
-                    if isinstance(query, str)
-                    else query
-                )
-            for i, (tmpfilePath, (content, expected)) in enumerate(
-                files.items(), start=1
-            ):
-                with open(tmpfilePath) as fh:
-                    self.assertEqual(
-                        fh.read(),
-                        expected,
-                        f"File {i} ({tmpfilePath!r}) has wrong content after hledger-edit",
+                }
+
+                for tmpfilePath, (content, expected) in files.items():
+                    with open(tmpfilePath, "w") as fh:
+                        fh.write(content)
+                        fh.flush()
+                with environment(
+                    EDITOR=editor,
+                    VISUAL=editor,
+                    HLEDGER_EDIT_LOGLEVEL="CRITICAL",
+                    HLEDGER_EDIT_NOTTY="yes",
+                ):
+                    hledger_utils.commands.edit.cli(
+                        list(
+                            itertools.chain.from_iterable(
+                                ("-f", f) for f in files
+                            )
+                        )
+                        + shlex.split(query)
+                        if isinstance(query, str)
+                        else query
                     )
+                for i, (tmpfilePath, (content, expected)) in enumerate(
+                    files.items(), start=1
+                ):
+                    with open(tmpfilePath) as fh:
+                        self.assertEqual(
+                            fh.read(),
+                            expected,
+                            f"File {i} ({tmpfilePath!r}) has wrong content after hledger-edit",
+                        )
+            if allowed_exitcodes is not None:
+                self.assertIn(cm.exception.code, allowed_exitcodes)
+        except SystemExit as e:
+            if allowed_exitcodes is not None:
+                self.assertIn(e.code, allowed_exitcodes)
         finally:
             try:
                 for tmpfilePath in files:
@@ -361,5 +385,29 @@ class HledgerEditTest(unittest.TestCase):
 2022-12-14  Sweets
     Mash        -3 €
     Cost:Sweets
+""",
+        )
+
+    def test_hledger_edit_forecasted_not_edited(self):
+        self.assert_hledger_edit(
+            content=f"""
+2022-12-12  Food
+    Cash        -10 €
+    Cost:Food
+
+~ monthly    Snacks
+    Cash                -20 €
+    Cost:Food:Snacks
+""",
+            editor="perl -pi -e 's|Snacks|SNAACKSSSSS!!!|g;s|10 €|30 €|g'",
+            query="--forecast",
+            expected_result=f"""
+2022-12-12  Food
+    Cash        -30 €
+    Cost:Food
+
+~ monthly    Snacks
+    Cash                -20 €
+    Cost:Food:Snacks
 """,
         )
